/**
 * @file CallbackDataTable.hpp
 *
 * @if NOTICE
 *
 * Copyright (c) proconX Pty Ltd. All rights reserved.
 *
 * THIS IS PROPRIETARY SOFTWARE AND YOU NEED A LICENSE TO USE OR REDISTRIBUTE.
 *
 * THIS SOFTWARE IS PROVIDED BY PROCONX AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL PROCONX OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * @endif
 */


#ifndef _CALLBACKDATATABLE_H_INCLUDED
#define _CALLBACKDATATABLE_H_INCLUDED


// Package header
#include "MbusSlaveCexports.h"
#include "MbusDataTableInterface.hpp"


/*****************************************************************************
 * Callback datatable class declaration
 *****************************************************************************/

class CallbackDataTable: public MbusDataTableInterface
{

  public:

   CallbackDataTable(
      ReadHoldingRegistersCbType  readHoldingRegistersCbPtr,
      WriteHoldingRegistersCbType writeHoldingRegistersCbPtr,
      ReadInputRegistersCbType    readInputRegistersCbPtr,
      ReadCoilsCbType             readCoilsCbPtr,
      WriteCoilsCbType            writeCoilsCbPtr,
      ReadInputDiscretesCbType    readInputDiscretesCbPtr,
      ReadEnronRegistersCbType    readEnronRegistersCbPtr,
      WriteEnronRegistersCbType   writeEnronRegistersCbPtr,
      ReadExceptionStatusCbType   readExceptionStatusCbPtr,
      TimeOutHandlerCbType        timeOutHandlerCbPtr
      )
   {
      this->readHoldingRegistersCbPtr  = readHoldingRegistersCbPtr;
      this->writeHoldingRegistersCbPtr = writeHoldingRegistersCbPtr;
      this->readInputRegistersCbPtr    = readInputRegistersCbPtr;
      this->readCoilsCbPtr             = readCoilsCbPtr;
      this->writeCoilsCbPtr            = writeCoilsCbPtr;
      this->readInputDiscretesCbPtr    = readInputDiscretesCbPtr;
      this->readEnronRegistersCbPtr    = readEnronRegistersCbPtr;
      this->writeEnronRegistersCbPtr   = writeEnronRegistersCbPtr;
      this->readExceptionStatusCbPtr   = readExceptionStatusCbPtr;
      this->timeOutHandlerCbPtr        = timeOutHandlerCbPtr;
   }


   int readHoldingRegistersTable(int startRef, short regArr[], int refCnt)
   {
      if (!readHoldingRegistersCbPtr)
         return 0;
      else
         return (*readHoldingRegistersCbPtr)(startRef, regArr, refCnt);
   }


   int writeHoldingRegistersTable(int startRef, const short regArr[], int refCnt)
   {
      if (!writeHoldingRegistersCbPtr)
         return 0;
      else
         return (*writeHoldingRegistersCbPtr)(startRef, regArr, refCnt);
   }


   int readInputRegistersTable(int startRef, short regArr[], int refCnt)
   {
      if (!readInputRegistersCbPtr)
         return 0;
      else
         return (*readInputRegistersCbPtr)(startRef, regArr, refCnt);
   }


   int readCoilsTable(int startRef, char bitArr[], int refCnt)
   {
      if (!readCoilsCbPtr)
         return 0;
      else
         return (*readCoilsCbPtr)(startRef, bitArr, refCnt);
   }


   int writeCoilsTable(int startRef, const char bitArr[], int refCnt)
   {
      if (!writeCoilsCbPtr)
         return 0;
      else
         return (*writeCoilsCbPtr)(startRef, bitArr, refCnt);
   }


   int readInputDiscretesTable(int startRef, char bitArr[], int refCnt)
   {
      if (!readInputDiscretesCbPtr)
         return 0;
      else
         return (*readInputDiscretesCbPtr)(startRef, bitArr, refCnt);
   }


   int readEnronRegistersTable(int startRef, long reg32Arr[], int refCnt)
   {
      if (!readEnronRegistersCbPtr)
         return 0;
      else
         return (*readEnronRegistersCbPtr)(startRef, reg32Arr, refCnt);
   }


   int writeEnronRegistersTable(int startRef, const long reg32Arr[], int refCnt)
   {
      if (!writeEnronRegistersCbPtr)
         return 0;
      else
         return (*writeEnronRegistersCbPtr)(startRef, reg32Arr, refCnt);
   }


   void timeOutHandler()
   {
      if (timeOutHandlerCbPtr)
         (*timeOutHandlerCbPtr)();
   }


   char readExceptionStatus()
   {
      if (!readExceptionStatusCbPtr)
         return 0;
      else
         return (*readExceptionStatusCbPtr)();
   }


  private:

   ReadHoldingRegistersCbType  readHoldingRegistersCbPtr;
   WriteHoldingRegistersCbType writeHoldingRegistersCbPtr;
   ReadInputRegistersCbType    readInputRegistersCbPtr;
   ReadCoilsCbType             readCoilsCbPtr;
   WriteCoilsCbType            writeCoilsCbPtr;
   ReadInputDiscretesCbType    readInputDiscretesCbPtr;
   ReadEnronRegistersCbType    readEnronRegistersCbPtr;
   WriteEnronRegistersCbType   writeEnronRegistersCbPtr;
   ReadExceptionStatusCbType   readExceptionStatusCbPtr;
   TimeOutHandlerCbType        timeOutHandlerCbPtr;

};


#endif // ifdef ..._H_INCLUDED
