/**
 * @file MbusAsciiSlaveProtocol.hpp
 *
 * @if NOTICE
 *
 * Copyright (c) proconX Pty Ltd. All rights reserved.
 *
 * THIS IS PROPRIETARY SOFTWARE AND YOU NEED A LICENSE TO USE OR REDISTRIBUTE.
 *
 * THIS SOFTWARE IS PROVIDED BY PROCONX AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL PROCONX OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * @endif
 */


#ifndef _MBUSACIISLAVEPROTOCOL_H_INCLUDED
#define _MBUSACIISLAVEPROTOCOL_H_INCLUDED

#ifndef __cplusplus
#  error Must use C++ to compile this module!
#endif

// Package header
#include "MbusSerialSlaveProtocol.hpp"


/*****************************************************************************
 * MbusAsciiSlaveProtocol class declaration
 *****************************************************************************/

/**
 * This class realises the server side of the Modbus ASCII slave protocol.
 *
 * This class provides functions to start-up and to execute the
 * Modbus ASCII server engine which includes opening and closing
 * of the serial port. Upon receipt of a valid master query the
 * server engine calls Data Provider methods to exchange data
 * with the user application. For a more detailed description
 * which Modbus data and control functions have been implemented
 * in the server engine see section
 * @ref mbusslave.
 *
 * It is possible to instantiate multiple instances for establishing
 * multiple connections on different serial ports, however they
 * must be executed in separate threads.
 *
 * @ingroup mbusslaveserial
 * @see mbusslave, mbusslaveserial
 * @see MbusDataTableInterface, MbusRtuSlaveProtocol, MbusTcpSlaveProtocol
 */
class
#if defined (_WINDLL) || defined(__DLL__)
   __declspec(dllexport)
#endif
MbusAsciiSlaveProtocol: public MbusSerialSlaveProtocol
{

  private:

     enum
     {
        // ASCII header length is 1 byte colon + 2 byte address/function in hex
        HDR_LEN = 1 + 2 * 2 ,
        // ASCII frame length is header + 1 byte lrc in hex encoding + cr + lf
        FRAME_LEN = HDR_LEN + 1 * 2 + 2,
        // ASCII exception message size is frame plus the code in hex encoding
        EXC_MSG_LEN = FRAME_LEN + 1 * 2,
        // ASCII max. message length is data size in hex + frame len
        MAX_MSG_SIZE = MAX_DATA_SIZE * 2 + FRAME_LEN
     };

   char bufferArr[MAX_DATA_SIZE + 2 + 1];
   char encodedBufArr[MAX_MSG_SIZE];


  public:

   MbusAsciiSlaveProtocol();

   int serverLoop();


  private:

   int replyMessage(int replyLen);


  private:

   // Disable default operator and copy constructor
   MbusAsciiSlaveProtocol &operator= (MbusAsciiSlaveProtocol &);
   MbusAsciiSlaveProtocol (const MbusAsciiSlaveProtocol &);

};


#endif // ifdef ..._H_INCLUDED
