/**
 * @file MbusSerialSlaveProtocol.hpp
 *
 * @if NOTICE
 *
 * Copyright (c) proconX Pty Ltd. All rights reserved.
 *
 * THIS IS PROPRIETARY SOFTWARE AND YOU NEED A LICENSE TO USE OR REDISTRIBUTE.
 *
 * THIS SOFTWARE IS PROVIDED BY PROCONX AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL PROCONX OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * @endif
 */


#ifndef _MBUSSERIALSLAVEPROTOCOL_H_INCLUDED
#define _MBUSSERIALSLAVEPROTOCOL_H_INCLUDED

#ifndef __cplusplus
#  error Must use C++ to compile this module!
#endif

// Platform header
#include <stdlib.h>

// Package header
#include "MbusSlaveServer.hpp"


/*****************************************************************************
 * Forward declaration
 *****************************************************************************/

class HmSerialPort;


/*****************************************************************************
 * MbusSerialSlaveProtocol class declaration
 *****************************************************************************/

/**
 * @brief This base class implements common functionality for both serial
 * Modbus protocols.
 *
 * These methods apply to RTU and ASCII protocol flavours via
 * inheritance.
 *
 * @see mbusslave, mbusslaveserial
 * @see MbusDataTableInterface,
 *      MbusRtuSlaveProtocol, MbusAsciiSlaveProtocol, MbusTcpSlaveProtocol
 */
class
#if defined (_WINDLL) || defined(__DLL__)
   __declspec(dllexport)
#endif
MbusSerialSlaveProtocol: public MbusSlaveServer
{

  public:

   enum
   {
      SER_DATABITS_7 = 7,    ///< 7 data bits
      SER_DATABITS_8 = 8     ///< 8 data bits
   };

   enum
   {
      SER_STOPBITS_1 = 1,    ///< 1 stop bit
      SER_STOPBITS_2 = 2     ///< 2 stop bits
   };

   enum
   {
      SER_PARITY_NONE = 0,   ///< No parity
      SER_PARITY_EVEN = 2,   ///< Even parity
      SER_PARITY_ODD = 1     ///< Odd parity
   };


  protected:

   HmSerialPort &serialPort;

   enum
   {
      SER_RS232, // RS232 mode w/o RTS/CTS handshake
      SER_RS485  // RS485 mode: RTS enables/disables transmitter
   };
   int serialMode;
   int rtsDelay;
   int masterIsConnected;
   volatile unsigned long underRunCounter;
   volatile unsigned long overRunCounter;
   volatile unsigned long crcErrorCounter;
   volatile unsigned long lineErrorCounter;
   volatile unsigned long frameErrorCounter;
   volatile unsigned long apiErrorCounter;

   MbusSerialSlaveProtocol();

   ~MbusSerialSlaveProtocol();


  public:

   virtual int startupServer(const TCHAR * const portName,
                             long baudRate, int dataBits,
                             int stopBits, int parity);

   void shutdownServer();

   int isStarted();

   int getConnectionStatus();

   virtual int enableRs485Mode(int rtsDelay);


  private:

   // Disable default operator and copy constructor
   MbusSerialSlaveProtocol &operator= (MbusSerialSlaveProtocol &);
   MbusSerialSlaveProtocol (const MbusSerialSlaveProtocol &);

};


#endif // ifdef ..._H_INCLUDED
