/**
 * @file MbusSlaveCexports.h
 *
 * @if NOTICE
 *
 * Copyright (c) proconX Pty Ltd. All rights reserved.
 *
 * THIS IS PROPRIETARY SOFTWARE AND YOU NEED A LICENSE TO USE OR REDISTRIBUTE.
 *
 * THIS SOFTWARE IS PROVIDED BY PROCONX AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL PROCONX OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * @endif
 */


#ifndef _MBUSSLAVECEXPORTS_H_INCLUDED
#define _MBUSSLAVECEXPORTS_H_INCLUDED


/* Platform header */
#include <stdlib.h>
#if defined(_WIN32) || defined(__WIN32__) || defined(_WIN32_WCE)
#  include <windows.h> /* Required for CE which defined __stdcall in it's own way... */
#  define STDCALL   __stdcall
#else
#  define STDCALL
#endif
#if defined (_WINDLL) || defined(__DLL__)
#  define EXPORT    __declspec(dllexport)
#else
#  define EXPORT
#endif

/* UNICODE support (MS VC++, Win CE) */
#if defined (_UNICODE) || defined(UNICODE)
#  ifndef _TCHAR_DEFINED
      typedef wchar_t TCHAR;
#     define _TCHAR_DEFINED
#  endif
#else
#  ifndef _TCHAR_DEFINED
      typedef char TCHAR;
#     define _TCHAR_DEFINED
#  endif
#endif

/* Package header */
#include "BusProtocolErrors.h"


/*****************************************************************************
 * Type definitions
 *****************************************************************************/

typedef void * MbusSlaveServerHdl;


/*****************************************************************************
 * Callback function type defs
 *****************************************************************************/

typedef int (STDCALL *ReadHoldingRegistersCbType)(int startRef, short regArr[], int refCnt);

typedef int (STDCALL *WriteHoldingRegistersCbType)(int startRef, const short regArr[], int refCnt);

typedef int (STDCALL *ReadInputRegistersCbType)(int startRef, short regArr[], int refCnt);

typedef int (STDCALL *ReadCoilsCbType)(int startRef,char bitArr[], int refCnt);

typedef int (STDCALL *WriteCoilsCbType)(int startRef, const char bitArr[], int refCnt);

typedef int (STDCALL *ReadInputDiscretesCbType)(int startRef, char bitArr[], int refCnt);

typedef int (STDCALL *ReadEnronRegistersCbType)(int startRef, long reg32Arr[], int refCnt);

typedef int (STDCALL *WriteEnronRegistersCbType)(int startRef, const long reg32Arr[], int refCnt);

typedef char (STDCALL *ReadExceptionStatusCbType)();

typedef void (STDCALL *TimeOutHandlerCbType)();


/*****************************************************************************
 * Prototypes
 *****************************************************************************/

#ifdef __cplusplus
extern "C" {
#endif


/*****************************************************************************
 * C wrapper functions to create and destroy protocol objects
 *****************************************************************************/

EXPORT MbusSlaveServerHdl STDCALL mbusSlave_createTcpProtocol(void);

EXPORT MbusSlaveServerHdl STDCALL mbusSlave_createRtuProtocol(void);

EXPORT MbusSlaveServerHdl STDCALL mbusSlave_createAsciiProtocol(void);

EXPORT void STDCALL mbusSlave_destroyProtocol(MbusSlaveServerHdl mbusHdl);


/*****************************************************************************
 * C wrapper functions for Server routines
 *****************************************************************************/

EXPORT int STDCALL
mbusSlave_configureCallbacks(MbusSlaveServerHdl mbusHdl, int slaveAddr,
                             ReadHoldingRegistersCbType  readHoldingRegistersCbPtr,
                             WriteHoldingRegistersCbType writeHoldingRegistersCbPtr,
                             ReadInputRegistersCbType    readInputRegistersCbPtr,
                             ReadCoilsCbType             readCoilsCbPtr,
                             WriteCoilsCbType            writeCoilsCbPtr,
                             ReadInputDiscretesCbType    readInputDiscretesCbPtr,
                             ReadEnronRegistersCbType    readEnronRegistersCbPtr,
                             WriteEnronRegistersCbType   writeEnronRegistersCbPtr,
                             ReadExceptionStatusCbType   readExceptionStatusCbPtr,
                             TimeOutHandlerCbType        timeOutHandlerCbPtr
                             );

EXPORT int STDCALL
mbusSlave_serverLoop(MbusSlaveServerHdl mbusHdl);

EXPORT void STDCALL mbusSlave_shutdownServer(MbusSlaveServerHdl mbusHdl);

EXPORT int STDCALL mbusSlave_isStarted(MbusSlaveServerHdl mbusHdl);

EXPORT int STDCALL mbusSlave_getConnectionStatus(MbusSlaveServerHdl mbusHdl);


/*****************************************************************************
 * C wrapper functions for Protocol Configuration
 *****************************************************************************/

EXPORT int STDCALL
mbusSlave_setTimeout(MbusSlaveServerHdl mbusHdl, int timeOut);

EXPORT int STDCALL mbusSlave_getTimeout(MbusSlaveServerHdl mbusHdl);

EXPORT void STDCALL
mbusSlave_enableExceptionReplies(MbusSlaveServerHdl mbusHdl);

EXPORT void STDCALL
mbusSlave_disableExceptionReplies(MbusSlaveServerHdl mbusHdl);


/*****************************************************************************
* C wrapper functions for Transmission Statistic Functions
*****************************************************************************/

EXPORT long STDCALL
mbusSlave_getTotalCounter(MbusSlaveServerHdl mbusHdl);

EXPORT void STDCALL
mbusSlave_resetTotalCounter(MbusSlaveServerHdl mbusHdl);

EXPORT long STDCALL
mbusSlave_getSuccessCounter(MbusSlaveServerHdl mbusHdl);

EXPORT void STDCALL
mbusSlave_resetSuccessCounter(MbusSlaveServerHdl mbusHdl);


/*****************************************************************************
 * C wrapper functions for Utility routines
 *****************************************************************************/

EXPORT const TCHAR * STDCALL mbusSlave_getPackageVersion(void);

EXPORT const TCHAR * STDCALL mbusSlave_getErrorText(int errCode);


/*****************************************************************************
 * C wrapper functions for MbusTcpSlaveProtocol class
 *****************************************************************************/

EXPORT int STDCALL
mbusSlave_startupTcpServer(MbusSlaveServerHdl mbusHdl,
                           const char * const hostName);

EXPORT int STDCALL
mbusMaster_setTcpPort(MbusSlaveServerHdl mbusHdl, unsigned short portNo);

EXPORT unsigned short  STDCALL
mbusMaster_getTcpPort(MbusSlaveServerHdl mbusHdl);


/*****************************************************************************
 * C wrapper functions for MbusSerialSlaveProtocol class
 *****************************************************************************/

EXPORT int STDCALL
mbusSlave_startupSerialServer(MbusSlaveServerHdl mbusHdl,
                              const TCHAR * const portName,
                              long baudRate, int dataBits,
                              int stopBits, int parity);

EXPORT int STDCALL
mbusSlave_enableRs485Mode(MbusSlaveServerHdl mbusHdl, int rtsDelay);


#ifdef __cplusplus
}
#endif

#endif /* ifdef ..._H_INCLUDED */
